@@ -39,6 +39,7 @@
 #endif
 
 #include "vmware.h"
+#include "vmware_common.h"
 #include "xf86xv.h"
 #include "fourcc.h"
 #include "svga_escape.h"
@@ -46,6 +47,17 @@
 
 #include <X11/extensions/Xv.h>
 
+#ifndef HAVE_XORG_SERVER_1_5_0
+#include <xf86_ansic.h>
+#include <xf86_libc.h>
+#endif
+
+
+#define HAVE_FILLKEYHELPERDRAWABLE \
+    ((GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) >= 2) ||  \
+     ((GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) == 1) && \
+      (GET_ABI_MINOR(ABI_VIDEODRV_VERSION) >= 2)))
+
 #define MAKE_ATOM(a) MakeAtom(a, sizeof(a) - 1, TRUE)
 
 /*
@@ -154,7 +166,7 @@ struct VMWAREVideoRec {
    int                (*play)(ScrnInfoPtr, struct VMWAREVideoRec *,
                               short, short, short, short, short,
                               short, short, short, int, unsigned char*,
-                              short, short, RegionPtr);
+                              short, short, RegionPtr, DrawablePtr);
    /*
     * Offscreen memory region used to pass video data to the host.
     */
@@ -175,7 +187,7 @@ typedef VMWAREVideoRec *VMWAREVideoPtr;
 /*
  * Callback functions
  */
-#ifdef HAVE_XORG_SERVER_1_0_99_901
+#if (GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) >= 1)
 static int vmwareXvPutImage(ScrnInfoPtr pScrn, short src_x, short src_y,
                             short drw_x, short drw_y, short src_w, short src_h,
                             short drw_w, short drw_h, int image,
@@ -212,7 +224,8 @@ static int vmwareVideoInitStream(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
                                  short drw_y, short src_w, short src_h,
                                  short drw_w, short drw_h, int format,
                                  unsigned char *buf, short width,
-                                 short height, RegionPtr clipBoxes);
+                                 short height, RegionPtr clipBoxes,
+				 DrawablePtr draw);
 static int vmwareVideoInitAttributes(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
                                      int format, unsigned short width,
                                      unsigned short height);
@@ -221,7 +234,8 @@ static int vmwareVideoPlay(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
                            short drw_y, short src_w, short src_h,
                            short drw_w, short drw_h, int format,
                            unsigned char *buf, short width,
-                           short height, RegionPtr clipBoxes);
+                           short height, RegionPtr clipBoxes,
+			   DrawablePtr draw);
 static void vmwareVideoFlush(VMWAREPtr pVMWARE, uint32 streamId);
 static void vmwareVideoSetOneReg(VMWAREPtr pVMWARE, uint32 streamId,
                                  uint32 regId, uint32 value);
@@ -334,7 +348,7 @@ vmwareOffscreenAllocate(VMWAREPtr pVMWARE, uint32 size)
         return NULL;
     }
 
-    memptr = xalloc(sizeof(VMWAREOffscreenRec));
+    memptr = malloc(sizeof(VMWAREOffscreenRec));
     if (!memptr) {
         return NULL;
     }
@@ -393,7 +407,8 @@ vmwareOffscreenFree(VMWAREOffscreenPtr memptr)
  *-----------------------------------------------------------------------------
  */
 
-Bool vmwareVideoEnabled(VMWAREPtr pVMWARE)
+Bool
+vmwareVideoEnabled(VMWAREPtr pVMWARE)
 {
     return ((pVMWARE->vmwareCapability & SVGA_CAP_EXTENDED_FIFO) &&
             (pVMWARE->vmwareFIFO[SVGA_FIFO_CAPABILITIES] &
@@ -418,9 +433,10 @@ Bool vmwareVideoEnabled(VMWAREPtr pVMWARE)
  *-----------------------------------------------------------------------------
  */
 
-Bool vmwareVideoInit(ScreenPtr pScreen)
+Bool
+vmwareVideoInit(ScreenPtr pScreen)
 {
-    ScrnInfoPtr pScrn = infoFromScreen(pScreen);
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     XF86VideoAdaptorPtr *overlayAdaptors, *newAdaptors = NULL;
     XF86VideoAdaptorPtr newAdaptor = NULL;
     int numAdaptors;
@@ -441,7 +457,7 @@ Bool vmwareVideoInit(ScreenPtr pScreen)
         numAdaptors = 1;
         overlayAdaptors = &newAdaptor;
     } else {
-         newAdaptors = xalloc((numAdaptors + 1) *
+         newAdaptors = malloc((numAdaptors + 1) *
                               sizeof(XF86VideoAdaptorPtr*));
          if (!newAdaptors) {
             xf86XVFreeVideoAdaptorRec(newAdaptor);
@@ -461,7 +477,7 @@ Bool vmwareVideoInit(ScreenPtr pScreen)
     }
 
     if (newAdaptors) {
-        xfree(newAdaptors);
+        free(newAdaptors);
     }
 
     xf86DrvMsg(pScrn->scrnIndex, X_INFO,
@@ -486,9 +502,10 @@ Bool vmwareVideoInit(ScreenPtr pScreen)
  *-----------------------------------------------------------------------------
  */
 
-void vmwareVideoEnd(ScreenPtr pScreen)
+void
+vmwareVideoEnd(ScreenPtr pScreen)
 {
-    ScrnInfoPtr pScrn = infoFromScreen(pScreen);
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     VMWAREPtr pVMWARE = VMWAREPTR(pScrn);
     VMWAREVideoPtr pVid;
     int i;
@@ -502,6 +519,7 @@ void vmwareVideoEnd(ScreenPtr pScreen)
     pVid = (VMWAREVideoPtr) &pVMWARE->videoStreams[VMWARE_VID_NUM_PORTS];
     for (i = 0; i < VMWARE_VID_NUM_PORTS; ++i) {
         vmwareVideoEndStream(pScrn, &pVid[i]);
+	REGION_UNINIT(pScreen, &pVid[i].clipBoxes);
     }
 
     free(pVMWARE->videoStreams);
@@ -526,7 +544,8 @@ void vmwareVideoEnd(ScreenPtr pScreen)
  *-----------------------------------------------------------------------------
  */
 
-static XF86VideoAdaptorPtr vmwareVideoSetup(ScrnInfoPtr pScrn)
+static XF86VideoAdaptorPtr
+vmwareVideoSetup(ScrnInfoPtr pScrn)
 {
     VMWAREPtr pVMWARE = VMWAREPTR(pScrn);
     XF86VideoAdaptorPtr adaptor;
@@ -541,7 +560,7 @@ static XF86VideoAdaptorPtr vmwareVideoSetup(ScrnInfoPtr pScrn)
         VmwareLog(("Not enough memory\n"));
         return NULL;
     }
-    du = xcalloc(1, VMWARE_VID_NUM_PORTS *
+    du = calloc(1, VMWARE_VID_NUM_PORTS *
         (sizeof(DevUnion) + sizeof(VMWAREVideoRec)));
 
     if (!du) {
@@ -568,6 +587,7 @@ static XF86VideoAdaptorPtr vmwareVideoSetup(ScrnInfoPtr pScrn)
         pPriv[i].flags = SVGA_VIDEO_FLAG_COLORKEY;
         pPriv[i].colorKey = VMWARE_VIDEO_COLORKEY;
         pPriv[i].isAutoPaintColorkey = TRUE;
+	REGION_NULL(pScreen, &pPriv[i].clipBoxes);
         adaptor->pPortPrivates[i].ptr = &pPriv[i];
     }
     pVMWARE->videoStreams = du;
@@ -614,12 +634,14 @@ static XF86VideoAdaptorPtr vmwareVideoSetup(ScrnInfoPtr pScrn)
  *-----------------------------------------------------------------------------
  */
 
-static int vmwareVideoInitStream(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
-                                 short src_x, short src_y, short drw_x,
-                                 short drw_y, short src_w, short src_h,
-                                 short drw_w, short drw_h, int format,
-                                 unsigned char *buf, short width,
-                                 short height, RegionPtr clipBoxes)
+static int
+vmwareVideoInitStream(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
+                      short src_x, short src_y, short drw_x,
+                      short drw_y, short src_w, short src_h,
+                      short drw_w, short drw_h, int format,
+                      unsigned char *buf, short width,
+                      short height, RegionPtr clipBoxes,
+		      DrawablePtr draw)
 {
     VMWAREPtr pVMWARE = VMWAREPTR(pScrn);
     int i;
@@ -661,7 +683,20 @@ static int vmwareVideoInitStream(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
     REGION_COPY(pScrn->pScreen, &pVid->clipBoxes, clipBoxes);
 
     if (pVid->isAutoPaintColorkey) {
+	BoxPtr boxes = REGION_RECTS(&pVid->clipBoxes);
+	int nBoxes = REGION_NUM_RECTS(&pVid->clipBoxes);
+
+#if HAVE_FILLKEYHELPERDRAWABLE
+	xf86XVFillKeyHelperDrawable(draw, pVid->colorKey, clipBoxes);
+#else
         xf86XVFillKeyHelper(pScrn->pScreen, pVid->colorKey, clipBoxes);
+#endif
+	/**
+	 * Force update to paint the colorkey before the overlay flush.
+	 */
+
+	while(nBoxes--)
+	    vmwareSendSVGACmdUpdate(pVMWARE, boxes++);
     }
 
     VmwareLog(("Got offscreen region, offset %d, size %d "
@@ -669,7 +704,8 @@ static int vmwareVideoInitStream(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
                pVid->fbarea->offset, pVid->fbarea->size, pVid->size));
 
     return pVid->play(pScrn, pVid, src_x, src_y, drw_x, drw_y, src_w, src_h,
-                      drw_w, drw_h, format, buf, width, height, clipBoxes);
+                      drw_w, drw_h, format, buf, width, height, clipBoxes,
+		      draw);
 }
 
 
@@ -689,16 +725,17 @@ static int vmwareVideoInitStream(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
  *-----------------------------------------------------------------------------
  */
 
-static int vmwareVideoInitAttributes(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
-                                     int format, unsigned short width,
-                                     unsigned short height)
+static int
+vmwareVideoInitAttributes(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
+                          int format, unsigned short width,
+                          unsigned short height)
 {
     int size;
     VMWAREVideoFmtData *fmtData;
 
     TRACEPOINT
 
-    fmtData = xcalloc(1, sizeof(VMWAREVideoFmtData));
+    fmtData = calloc(1, sizeof(VMWAREVideoFmtData));
     if (!fmtData) {
         return -1;
     }
@@ -732,12 +769,14 @@ static int vmwareVideoInitAttributes(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
  *-----------------------------------------------------------------------------
  */
 
-static int vmwareVideoPlay(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
-                           short src_x, short src_y, short drw_x,
-                           short drw_y, short src_w, short src_h,
-                           short drw_w, short drw_h, int format,
-                           unsigned char *buf, short width,
-                           short height, RegionPtr clipBoxes)
+static int
+vmwareVideoPlay(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
+                short src_x, short src_y, short drw_x,
+                short drw_y, short src_w, short src_h,
+                short drw_w, short drw_h, int format,
+                unsigned char *buf, short width,
+                short height, RegionPtr clipBoxes,
+		DrawablePtr draw)
 {
     VMWAREPtr pVMWARE = VMWAREPTR(pScrn);
     uint32 *fifoItem;
@@ -779,7 +818,7 @@ static int vmwareVideoPlay(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
         vmwareStopVideo(pScrn, pVid, TRUE);
         return pVid->play(pScrn, pVid, src_x, src_y, drw_x, drw_y, src_w,
                           src_h, drw_w, drw_h, format, buf, width, height,
-                          clipBoxes);
+                          clipBoxes, draw);
     }
 
     pVid->size = size;
@@ -823,13 +862,27 @@ static int vmwareVideoPlay(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
         vmwareWriteWordToFIFO(pVMWARE, fifoItem[i]);
     }
 
-    /* 
+    /*
      *  Update the clipList and paint the colorkey, if required.
      */
-    if (!REGION_EQUAL(pScrn->pScreen, &pVid->clipBoxes, clipBoxes)) {
+    if (!vmwareIsRegionEqual(&pVid->clipBoxes, clipBoxes)) {
         REGION_COPY(pScrn->pScreen, &pVid->clipBoxes, clipBoxes);
         if (pVid->isAutoPaintColorkey) {
-            xf86XVFillKeyHelper(pScrn->pScreen, pVid->colorKey, clipBoxes);
+	    BoxPtr boxes = REGION_RECTS(&pVid->clipBoxes);
+	    int nBoxes = REGION_NUM_RECTS(&pVid->clipBoxes);
+
+#if HAVE_FILLKEYHELPERDRAWABLE
+	    xf86XVFillKeyHelperDrawable(draw, pVid->colorKey, clipBoxes);
+#else
+	    xf86XVFillKeyHelper(pScrn->pScreen, pVid->colorKey, clipBoxes);
+#endif
+	    /**
+	     * Force update to paint the colorkey before the overlay flush.
+	     */
+
+	    while(nBoxes--)
+		vmwareSendSVGACmdUpdate(pVMWARE, boxes++);
+
         }
     }
 
@@ -858,7 +911,8 @@ static int vmwareVideoPlay(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid,
  *-----------------------------------------------------------------------------
  */
 
-static void vmwareVideoFlush(VMWAREPtr pVMWARE, uint32 streamId)
+static void
+vmwareVideoFlush(VMWAREPtr pVMWARE, uint32 streamId)
 {
     struct PACKED _body {
         uint32 escape;
@@ -904,8 +958,9 @@ static void vmwareVideoFlush(VMWAREPtr pVMWARE, uint32 streamId)
  *-----------------------------------------------------------------------------
  */
 
-static void vmwareVideoSetOneReg(VMWAREPtr pVMWARE, uint32 streamId,
-                                 uint32 regId, uint32 value)
+static void
+vmwareVideoSetOneReg(VMWAREPtr pVMWARE, uint32 streamId,
+                     uint32 regId, uint32 value)
 {
     struct PACKED _item {
         uint32 regId;
@@ -960,7 +1015,8 @@ static void vmwareVideoSetOneReg(VMWAREPtr pVMWARE, uint32 streamId,
  *-----------------------------------------------------------------------------
  */
 
-static void vmwareVideoEndStream(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid)
+static void
+vmwareVideoEndStream(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid)
 {
     uint32 id, colorKey, flags;
     Bool isAutoPaintColorkey;
@@ -1018,19 +1074,21 @@ static void vmwareVideoEndStream(ScrnInfoPtr pScrn, VMWAREVideoPtr pVid)
  *-----------------------------------------------------------------------------
  */
 
-#ifdef HAVE_XORG_SERVER_1_0_99_901
-static int vmwareXvPutImage(ScrnInfoPtr pScrn, short src_x, short src_y,
-                            short drw_x, short drw_y, short src_w, short src_h,
-                            short drw_w, short drw_h, int format,
-                            unsigned char *buf, short width, short height,
-                            Bool sync, RegionPtr clipBoxes, pointer data,
-                            DrawablePtr dst)
+#if (GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) >= 1)
+static int
+vmwareXvPutImage(ScrnInfoPtr pScrn, short src_x, short src_y,
+                 short drw_x, short drw_y, short src_w, short src_h,
+                 short drw_w, short drw_h, int format,
+                 unsigned char *buf, short width, short height,
+                 Bool sync, RegionPtr clipBoxes, pointer data,
+                 DrawablePtr dst)
 #else
-static int vmwareXvPutImage(ScrnInfoPtr pScrn, short src_x, short src_y,
-                            short drw_x, short drw_y, short src_w, short src_h,
-                            short drw_w, short drw_h, int format,
-                            unsigned char *buf, short width, short height,
-                            Bool sync, RegionPtr clipBoxes, pointer data)
+static int
+vmwareXvPutImage(ScrnInfoPtr pScrn, short src_x, short src_y,
+                 short drw_x, short drw_y, short src_w, short src_h,
+                 short drw_w, short drw_h, int format,
+                 unsigned char *buf, short width, short height,
+                 Bool sync, RegionPtr clipBoxes, pointer data)
 #endif
 {
     VMWAREPtr pVMWARE = VMWAREPTR(pScrn);
@@ -1042,8 +1100,15 @@ static int vmwareXvPutImage(ScrnInfoPtr pScrn, short src_x, short src_y,
         return XvBadAlloc;
     }
 
+#if (GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) >= 1)
     return pVid->play(pScrn, pVid, src_x, src_y, drw_x, drw_y, src_w, src_h,
-                      drw_w, drw_h, format, buf, width, height, clipBoxes);
+                      drw_w, drw_h, format, buf, width, height, clipBoxes,
+		      dst);
+#else
+    return pVid->play(pScrn, pVid, src_x, src_y, drw_x, drw_y, src_w, src_h,
+                      drw_w, drw_h, format, buf, width, height, clipBoxes,
+		      NULL);
+#endif
 }
 
 
@@ -1066,7 +1131,8 @@ static int vmwareXvPutImage(ScrnInfoPtr pScrn, short src_x, short src_y,
  *-----------------------------------------------------------------------------
  */
 
-static void vmwareStopVideo(ScrnInfoPtr pScrn, pointer data, Bool Cleanup)
+static void
+vmwareStopVideo(ScrnInfoPtr pScrn, pointer data, Bool Cleanup)
 {
     VMWAREVideoPtr pVid = data;
     VMWAREPtr pVMWARE = VMWAREPTR(pScrn);
@@ -1075,6 +1141,9 @@ static void vmwareStopVideo(ScrnInfoPtr pScrn, pointer data, Bool Cleanup)
     if (!vmwareVideoEnabled(pVMWARE)) {
         return;
     }
+
+    REGION_EMPTY(pScrn->pScreen, &pVid->clipBoxes);
+
     if (!Cleanup) {
         VmwareLog(("vmwareStopVideo: Cleanup is FALSE.\n"));
         return;
@@ -1107,10 +1176,10 @@ static void vmwareStopVideo(ScrnInfoPtr pScrn, pointer data, Bool Cleanup)
  *-----------------------------------------------------------------------------
  */
 
-static int vmwareQueryImageAttributes(ScrnInfoPtr pScrn, int format,
-                                      unsigned short *width,
-                                      unsigned short *height, int *pitches,
-                                      int *offsets)
+static int
+vmwareQueryImageAttributes(ScrnInfoPtr pScrn, int format,
+                           unsigned short *width, unsigned short *height,
+                           int *pitches, int *offsets)
 {
     INT32 size, tmp;
 
@@ -1184,8 +1253,9 @@ static int vmwareQueryImageAttributes(ScrnInfoPtr pScrn, int format,
  *-----------------------------------------------------------------------------
  */
 
-static int vmwareSetPortAttribute(ScrnInfoPtr pScrn, Atom attribute,
-                                  INT32 value, pointer data)
+static int
+vmwareSetPortAttribute(ScrnInfoPtr pScrn, Atom attribute,
+                       INT32 value, pointer data)
 {
     VMWAREVideoPtr pVid = (VMWAREVideoPtr) data;
     Atom xvColorKey = MAKE_ATOM("XV_COLORKEY");
@@ -1223,8 +1293,9 @@ static int vmwareSetPortAttribute(ScrnInfoPtr pScrn, Atom attribute,
  *-----------------------------------------------------------------------------
  */
 
-static int vmwareGetPortAttribute(ScrnInfoPtr pScrn, Atom attribute,
-                                  INT32 *value, pointer data)
+static int
+vmwareGetPortAttribute(ScrnInfoPtr pScrn, Atom attribute,
+                       INT32 *value, pointer data)
 {
     VMWAREVideoPtr pVid = (VMWAREVideoPtr) data;
     Atom xvColorKey = MAKE_ATOM("XV_COLORKEY");
@@ -1267,10 +1338,11 @@ static int vmwareGetPortAttribute(ScrnInfoPtr pScrn, Atom attribute,
  *-----------------------------------------------------------------------------
  */
 
-static void vmwareQueryBestSize(ScrnInfoPtr pScrn, Bool motion,
-                                short vid_w, short vid_h, short drw_w,
-                                short drw_h, unsigned int *p_w,
-                                unsigned int *p_h, pointer data)
+static void
+vmwareQueryBestSize(ScrnInfoPtr pScrn, Bool motion,
+                    short vid_w, short vid_h, short drw_w,
+                    short drw_h, unsigned int *p_w,
+                    unsigned int *p_h, pointer data)
 {
     *p_w = (drw_w + 1) & ~1;
     *p_h = drw_h;
